# coding: utf-8

"""
    Seeq REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 59.0.1-SNAPSHOT
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from pprint import pformat
from six import iteritems
import re


class DatasourcesStatusOutputV1(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """


    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'agents': 'list[AgentStatusOutputV1]',
        'connected_connections_count': 'int',
        'connected_datasources_count': 'int',
        'datasources': 'list[DatasourceSummaryStatusOutputV1]',
        'total_connections_count': 'int',
        'total_datasources_count': 'int'
    }

    attribute_map = {
        'agents': 'agents',
        'connected_connections_count': 'connectedConnectionsCount',
        'connected_datasources_count': 'connectedDatasourcesCount',
        'datasources': 'datasources',
        'total_connections_count': 'totalConnectionsCount',
        'total_datasources_count': 'totalDatasourcesCount'
    }

    def __init__(self, agents=None, connected_connections_count=None, connected_datasources_count=None, datasources=None, total_connections_count=None, total_datasources_count=None):
        """
        DatasourcesStatusOutputV1 - a model defined in Swagger
        """

        self._agents = None
        self._connected_connections_count = None
        self._connected_datasources_count = None
        self._datasources = None
        self._total_connections_count = None
        self._total_datasources_count = None

        if agents is not None:
          self.agents = agents
        if connected_connections_count is not None:
          self.connected_connections_count = connected_connections_count
        if connected_datasources_count is not None:
          self.connected_datasources_count = connected_datasources_count
        if datasources is not None:
          self.datasources = datasources
        if total_connections_count is not None:
          self.total_connections_count = total_connections_count
        if total_datasources_count is not None:
          self.total_datasources_count = total_datasources_count

    @property
    def agents(self):
        """
        Gets the agents of this DatasourcesStatusOutputV1.
        The status for all agents accessible by the user. Provided only when the required level of detail is 'Complete'

        :return: The agents of this DatasourcesStatusOutputV1.
        :rtype: list[AgentStatusOutputV1]
        """
        return self._agents

    @agents.setter
    def agents(self, agents):
        """
        Sets the agents of this DatasourcesStatusOutputV1.
        The status for all agents accessible by the user. Provided only when the required level of detail is 'Complete'

        :param agents: The agents of this DatasourcesStatusOutputV1.
        :type: list[AgentStatusOutputV1]
        """

        self._agents = agents

    @property
    def connected_connections_count(self):
        """
        Gets the connected_connections_count of this DatasourcesStatusOutputV1.
        The number of connections in status 'Connected'. Provided regardless of the level of detail requested

        :return: The connected_connections_count of this DatasourcesStatusOutputV1.
        :rtype: int
        """
        return self._connected_connections_count

    @connected_connections_count.setter
    def connected_connections_count(self, connected_connections_count):
        """
        Sets the connected_connections_count of this DatasourcesStatusOutputV1.
        The number of connections in status 'Connected'. Provided regardless of the level of detail requested

        :param connected_connections_count: The connected_connections_count of this DatasourcesStatusOutputV1.
        :type: int
        """
        if connected_connections_count is None:
            raise ValueError("Invalid value for `connected_connections_count`, must not be `None`")

        self._connected_connections_count = connected_connections_count

    @property
    def connected_datasources_count(self):
        """
        Gets the connected_datasources_count of this DatasourcesStatusOutputV1.
        The number of connected datasources. Provided regardless of the level of detail requested

        :return: The connected_datasources_count of this DatasourcesStatusOutputV1.
        :rtype: int
        """
        return self._connected_datasources_count

    @connected_datasources_count.setter
    def connected_datasources_count(self, connected_datasources_count):
        """
        Sets the connected_datasources_count of this DatasourcesStatusOutputV1.
        The number of connected datasources. Provided regardless of the level of detail requested

        :param connected_datasources_count: The connected_datasources_count of this DatasourcesStatusOutputV1.
        :type: int
        """
        if connected_datasources_count is None:
            raise ValueError("Invalid value for `connected_datasources_count`, must not be `None`")

        self._connected_datasources_count = connected_datasources_count

    @property
    def datasources(self):
        """
        Gets the datasources of this DatasourcesStatusOutputV1.
        The status for all datasources accessible by the user. Available only when the required level of detail is 'Complete' or 'Summary'

        :return: The datasources of this DatasourcesStatusOutputV1.
        :rtype: list[DatasourceSummaryStatusOutputV1]
        """
        return self._datasources

    @datasources.setter
    def datasources(self, datasources):
        """
        Sets the datasources of this DatasourcesStatusOutputV1.
        The status for all datasources accessible by the user. Available only when the required level of detail is 'Complete' or 'Summary'

        :param datasources: The datasources of this DatasourcesStatusOutputV1.
        :type: list[DatasourceSummaryStatusOutputV1]
        """

        self._datasources = datasources

    @property
    def total_connections_count(self):
        """
        Gets the total_connections_count of this DatasourcesStatusOutputV1.
        The total number of connections expected to be in the status 'Connected'. Provided regardless of the level of detail requested

        :return: The total_connections_count of this DatasourcesStatusOutputV1.
        :rtype: int
        """
        return self._total_connections_count

    @total_connections_count.setter
    def total_connections_count(self, total_connections_count):
        """
        Sets the total_connections_count of this DatasourcesStatusOutputV1.
        The total number of connections expected to be in the status 'Connected'. Provided regardless of the level of detail requested

        :param total_connections_count: The total_connections_count of this DatasourcesStatusOutputV1.
        :type: int
        """
        if total_connections_count is None:
            raise ValueError("Invalid value for `total_connections_count`, must not be `None`")

        self._total_connections_count = total_connections_count

    @property
    def total_datasources_count(self):
        """
        Gets the total_datasources_count of this DatasourcesStatusOutputV1.
        The total number of datasources expected to be connected. Provided regardless of the level of detail requested

        :return: The total_datasources_count of this DatasourcesStatusOutputV1.
        :rtype: int
        """
        return self._total_datasources_count

    @total_datasources_count.setter
    def total_datasources_count(self, total_datasources_count):
        """
        Sets the total_datasources_count of this DatasourcesStatusOutputV1.
        The total number of datasources expected to be connected. Provided regardless of the level of detail requested

        :param total_datasources_count: The total_datasources_count of this DatasourcesStatusOutputV1.
        :type: int
        """
        if total_datasources_count is None:
            raise ValueError("Invalid value for `total_datasources_count`, must not be `None`")

        self._total_datasources_count = total_datasources_count

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        if not isinstance(other, DatasourcesStatusOutputV1):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



from __future__ import annotations

import types
from typing import Optional, Union

import pandas as pd
from seeq import spy
from seeq.sdk import *
from seeq.spy import _common, _login, _metadata
from seeq.spy._errors import *
from seeq.spy._session import Session
from seeq.spy._status import Status
from seeq.spy.acl import _pull
from seeq.spy.workbooks import Item


def push(items: Union[pd.DataFrame, dict, list, str, Item], acl: Union[pd.DataFrame, dict, list], *,
         replace: bool = False, disable_inheritance: bool = None, errors: Optional[str] = None, quiet: bool = False,
         status: Optional[Status] = None, session: Optional[Session] = None) -> pd.DataFrame:
    """
    Pushes new access control entries against a set of items as specified
    by their IDs. The most common way to invoke this command is directly
    after having done a spy.search() or spy.push() and produced a DataFrame
    full of items to work with.

    Parameters
    ----------
    items : {pandas.DataFrame, dict, list, str, Item}
        The item IDs to push against. This argument can take the following
        form:

        - a DataFrame with an "ID" column
        - a single dict with an "ID" key
        - a list of dicts with "ID" keys
        - a single Workbook object
        - a list of Workbook objects
        - a single string representing the ID of the item
        - a list of strings representing the IDs of the items
        - a SPy Item instance

    acl : {pandas.DataFrame, dict, list}
        The Access Control List can be either a Pandas DataFrame
        a list of dicts, or just a dict. The dict or DataFrame rows must
        consist of either 'ID' OR ('Name' or 'Username') to specify the
        identity and then a 'Read', 'Write' and/or 'Manage' entry that is
        a boolean True or False value.

    replace : {bool}, optional, default False
        - If False, then existing access control entries will not be disturbed
          but new entries will be added.
        - If True, then existing access control entries will be removed
          and replaced with the entries from the acl argument.

    disable_inheritance : {bool}, optional, default None
        - If None, then the inheritance will remain unchanged. (This is the
          default behavior.)
        - If False, then the ACL inherits from either the datasource or the
          workbook (for signals/scalars/conditions/assets/metrics). Workbooks
          inherit their ACLs from their parent folder.
        - If True, then the ACL will no longer inherit from the parent.

    errors : {'raise', 'catalog'}, default 'raise'
        If 'raise', any errors encountered will cause an exception. If
        'catalog', errors will be added to a 'Push Result' column in the
        status.df DataFrame.

    quiet : bool, default False
        If True, suppresses progress output. Note that when status is
        provided, the quiet setting of the Status object that is passed
        in takes precedent.

    status : spy.Status, optional
        If specified, the supplied Status object will be updated as the command
        progresses. It gets filled in with the same information you would see
        in Jupyter in the blue/green/red table below your code while the
        command is executed. The table itself is accessible as a DataFrame via
        the status.df property.

    session : spy.Session, optional
        If supplied, the Session object (and its Options) will be used to
        store the login session state. This is useful to log in to different
        Seeq servers at the same time or with different credentials.

    Returns
    -------
    pandas.DataFrame
        A DataFrame with the metadata for the items pushed, along with any
        errors (in the 'Push Result' column).

        Additionally, the following properties are stored on the "spy"
        attribute of the output DataFrame:

        =================== ===================================================
        Property            Description
        =================== ===================================================
        func                A str value of 'spy.acl.push'
        kwargs              A dict with the values of the input parameters
                            passed to spy.acl.push to get the output DataFrame
        status              A spy.Status object with the status of the
                            spy.acl.push call
        =================== ===================================================

    Examples
    --------
    Search for signals with the name 'Hydrocracker' and add a single ACL for Walter:

    >>> search_results = spy.search({'Name': 'Hydrocracker'})
    >>> push_results = spy.acl.push(search_results, {'Username': 'walter.reed@va.com', 'Read': True})

    Add multiple ACLs -- one for Reed Abook (a user) and one for the Everyone group:

    >>> search_results = spy.search({'Name': 'Hydrocracker'})
    >>> push_results = spy.acl.push(search_results, [{
    >>>                                 'Username': 'reed.abook@seeq.com',
    >>>                                 'Read': True,
    >>>                                 'Write': True,
    >>>                                 'Manage': True
    >>>                             }, {
    >>>                                 'Name': 'Everyone',
    >>>                                 'Type': 'UserGroup',
    >>>                                 'Read': True
    >>>                             }])
    """
    input_args = _common.validate_argument_types([
        (items, 'items', (pd.DataFrame, list, dict, str, Item)),
        (acl, 'acl', (pd.DataFrame, list, dict)),
        (replace, 'replace', bool),
        (disable_inheritance, 'disable_inheritance', bool),
        (errors, 'errors', str),
        (quiet, 'quiet', bool),
        (status, 'status', Status),
        (session, 'session', Session)
    ])

    status = Status.validate(status, quiet, errors)
    session = Session.validate(session)
    _login.validate_login(session, status)

    users_api = UsersApi(session.client)

    items = _pull.items_to_data_frame(items)

    acl = _common.force_to_data_frame(acl)

    if not isinstance(acl, pd.DataFrame):
        raise SPyTypeError('acl argument is not a DataFrame or DataFrame-like object')

    status.df = pd.DataFrame({
        'Count': pd.Series([0], dtype=int)
    })

    push_result_df = items.copy()
    push_result_df.drop(columns=[c for c in _metadata.RESERVED_SPY_STATUS_COLUMN_NAMES if c in push_result_df],
                        inplace=True)

    count = 0
    timer = _common.timer_start()
    for acl_index, acl_row in acl.iterrows():
        if not _common.present(acl_row, 'Read'):
            acl.at[acl_index, 'Read'] = False
        if not _common.present(acl_row, 'Write'):
            acl.at[acl_index, 'Write'] = False
        if not _common.present(acl_row, 'Manage'):
            acl.at[acl_index, 'Manage'] = False

        # The UI implicitly sets Read/Write if a higher permission is granted, so we'll do the same here
        if acl.at[acl_index, 'Manage']:
            acl.at[acl_index, 'Write'] = True
        if acl.at[acl_index, 'Write']:
            acl.at[acl_index, 'Read'] = True

        if not _common.present(acl_row, 'ID'):
            if not _common.present(acl_row, 'Name') and not _common.present(acl_row, 'Username'):
                raise SPyValueError(
                    f'Access Control DataFrame must include value in either ID or Name or Username column:\n'
                    f'{acl_row}')

            matches = list()
            query = _common.get(acl_row, 'Username') if _common.present(acl_row, 'Username') else \
                _common.get(acl_row, 'Name')

            identities = users_api.autocomplete_users_and_groups(query=query, limit=100000)

            for identity in identities.items:  # type: IdentityPreviewV1
                if (_common.present(acl_row, 'Name') and
                        _insensitive_not_equal(identity.name, _common.get(acl_row, 'Name'))):
                    continue

                if (_common.present(acl_row, 'Username') and
                        _insensitive_not_equal(identity.username, _common.get(acl_row, 'Username'))):
                    continue

                if (_common.present(acl_row, 'Type') and
                        _insensitive_not_equal(identity.type, _common.get(acl_row, 'Type'))):
                    continue

                if (_common.present(acl_row, 'Directory') and
                        _insensitive_not_equal(identity.datasource.name, _common.get(acl_row, 'Directory'))):
                    continue

                matches.append(identity)

            if len(matches) == 0:
                raise SPyValueError(
                    f'Could not find identity "{query}" for Access Control entry:\n{acl_row}')
            elif len(matches) > 1:
                raise SPyValueError(
                    f'Multiple matches found for "{query}" for Access Control entry:\n{acl_row}\n'
                    'Narrow down by supplying "Name", "Username", "Type" and/or "Directory" columns (if you '
                    "haven't already)")
            else:
                acl.at[acl_index, 'ID'] = matches[0].id
                acl.at[acl_index, 'Name'] = matches[0].name
                acl.at[acl_index, 'Directory'] = matches[0].datasource.name

    status.df['Time'] = _common.timer_elapsed(timer)
    status.update('Pushing ACLs...', Status.RUNNING)

    for metadata_index, metadata_row in push_result_df.iterrows():
        count += 1
        status.df['Count'] = count
        try:
            item_id = metadata_row['ID']

            _metadata.push_access_control(session, item_id, acl, replace,
                                          disable_permission_inheritance=disable_inheritance)

            status.df['Time'] = _common.timer_elapsed(timer)
            status.update('Pushing ACLs...', Status.RUNNING)
            push_result_df.at[metadata_index, 'Push Result'] = 'Success'

        except Exception as e:
            if status.errors == 'raise':
                raise

            push_result_df.at[metadata_index, 'Push Result'] = _common.format_exception(e)

    if push_result_df['Push Result'].drop_duplicates().tolist() != ['Success']:
        status.update('Pushed ACLs with errors, see "Push Result" column in returned DataFrame.', Status.FAILURE)
    else:
        status.update('Pushed ACLs successfully.', Status.SUCCESS)

    push_df_properties = types.SimpleNamespace(
        func='spy.acl.push',
        kwargs=input_args,
        status=status)

    _common.put_properties_on_df(push_result_df, push_df_properties)

    return push_result_df


def _insensitive_not_equal(a: Optional[str], b: Optional[str]):
    if a is None and b is None:
        return False
    elif a is not None and b is None:
        return True
    elif a is None and b is not None:
        return True
    else:
        return a.lower() != b.lower()



from __future__ import annotations

import glob
import os
import tempfile
import zipfile

from seeq.base import system
from seeq.spy import _common
from seeq.spy._errors import *
from seeq.spy._status import Status
from seeq.spy.workbooks._template import Analysis, AnalysisTemplate, TopicTemplate
from seeq.spy.workbooks._workbook import Workbook, WorkbookList


def load(folder_or_zipfile, *, as_template_with_label: str = None) -> WorkbookList:
    """
    Loads a list of workbooks from a folder on disk into Workbook objects in
    memory.

    Parameters
    ----------
    folder_or_zipfile : str
        A folder or zip file on disk containing workbooks to be loaded. Note
        that any subfolder structure will work -- this function will scan for
        any subfolders that contain a Workbook.json file and assume they should
        be loaded.

    as_template_with_label : str
        Causes the workbooks to be loaded as templates (either AnalysisTemplate
        or TopicTemplate) with the label specified. See the Workbook Templates
        documentation notebook for more information about templates.
    """
    status = Status()

    _common.validate_argument_types([
        (folder_or_zipfile, 'folder_or_zipfile', str),
        (as_template_with_label, 'as_template_with_label', str)
    ])

    folder_or_zipfile = system.cleanse_path(folder_or_zipfile)

    try:
        if not os.path.exists(folder_or_zipfile):
            raise SPyRuntimeError('Folder/zipfile "%s" does not exist' % folder_or_zipfile)

        if folder_or_zipfile.lower().endswith('.zip'):
            with tempfile.TemporaryDirectory() as temp:
                with zipfile.ZipFile(folder_or_zipfile, "r") as z:
                    status.update('Unzipping "%s"' % folder_or_zipfile, Status.RUNNING)
                    z.extractall(temp)

                status.update('Loading from "%s"' % temp, Status.RUNNING)
                workbooks = _load_from_folder(temp)
        else:
            status.update('Loading from "%s"' % folder_or_zipfile, Status.RUNNING)
            workbooks = _load_from_folder(folder_or_zipfile)

        if as_template_with_label is not None:
            package = WorkbookList()
            for workbook in workbooks:
                package.append(_get_template_class(workbook)(as_template_with_label, workbook, package=package))
            workbooks = package

        status.update('Success', Status.SUCCESS)
        return workbooks

    except KeyboardInterrupt:
        status.update('Load canceled', Status.CANCELED)


def _get_template_class(workbook):
    return AnalysisTemplate if isinstance(workbook, Analysis) else TopicTemplate


def _load_from_folder(folder):
    workbook_json_files = glob.glob(os.path.join(folder, '**', 'Workbook.json'), recursive=True)

    workbooks = WorkbookList()
    for workbook_json_file in workbook_json_files:
        workbooks.append(Workbook.load(os.path.dirname(workbook_json_file)))

    return workbooks



# coding: utf-8

"""
    Seeq REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 59.0.1-SNAPSHOT
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from pprint import pformat
from six import iteritems
import re


class SyncProgress(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """


    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'asset_count': 'int',
        'asset_progress': 'int',
        'condition_count': 'int',
        'condition_progress': 'int',
        'relationship_count': 'int',
        'relationship_progress': 'int',
        'scalar_count': 'int',
        'scalar_progress': 'int',
        'signal_count': 'int',
        'signal_progress': 'int',
        'user_group_count': 'int',
        'user_group_progress': 'int'
    }

    attribute_map = {
        'asset_count': 'assetCount',
        'asset_progress': 'assetProgress',
        'condition_count': 'conditionCount',
        'condition_progress': 'conditionProgress',
        'relationship_count': 'relationshipCount',
        'relationship_progress': 'relationshipProgress',
        'scalar_count': 'scalarCount',
        'scalar_progress': 'scalarProgress',
        'signal_count': 'signalCount',
        'signal_progress': 'signalProgress',
        'user_group_count': 'userGroupCount',
        'user_group_progress': 'userGroupProgress'
    }

    def __init__(self, asset_count=None, asset_progress=None, condition_count=None, condition_progress=None, relationship_count=None, relationship_progress=None, scalar_count=None, scalar_progress=None, signal_count=None, signal_progress=None, user_group_count=None, user_group_progress=None):
        """
        SyncProgress - a model defined in Swagger
        """

        self._asset_count = None
        self._asset_progress = None
        self._condition_count = None
        self._condition_progress = None
        self._relationship_count = None
        self._relationship_progress = None
        self._scalar_count = None
        self._scalar_progress = None
        self._signal_count = None
        self._signal_progress = None
        self._user_group_count = None
        self._user_group_progress = None

        if asset_count is not None:
          self.asset_count = asset_count
        if asset_progress is not None:
          self.asset_progress = asset_progress
        if condition_count is not None:
          self.condition_count = condition_count
        if condition_progress is not None:
          self.condition_progress = condition_progress
        if relationship_count is not None:
          self.relationship_count = relationship_count
        if relationship_progress is not None:
          self.relationship_progress = relationship_progress
        if scalar_count is not None:
          self.scalar_count = scalar_count
        if scalar_progress is not None:
          self.scalar_progress = scalar_progress
        if signal_count is not None:
          self.signal_count = signal_count
        if signal_progress is not None:
          self.signal_progress = signal_progress
        if user_group_count is not None:
          self.user_group_count = user_group_count
        if user_group_progress is not None:
          self.user_group_progress = user_group_progress

    @property
    def asset_count(self):
        """
        Gets the asset_count of this SyncProgress.

        :return: The asset_count of this SyncProgress.
        :rtype: int
        """
        return self._asset_count

    @asset_count.setter
    def asset_count(self, asset_count):
        """
        Sets the asset_count of this SyncProgress.

        :param asset_count: The asset_count of this SyncProgress.
        :type: int
        """

        self._asset_count = asset_count

    @property
    def asset_progress(self):
        """
        Gets the asset_progress of this SyncProgress.

        :return: The asset_progress of this SyncProgress.
        :rtype: int
        """
        return self._asset_progress

    @asset_progress.setter
    def asset_progress(self, asset_progress):
        """
        Sets the asset_progress of this SyncProgress.

        :param asset_progress: The asset_progress of this SyncProgress.
        :type: int
        """

        self._asset_progress = asset_progress

    @property
    def condition_count(self):
        """
        Gets the condition_count of this SyncProgress.

        :return: The condition_count of this SyncProgress.
        :rtype: int
        """
        return self._condition_count

    @condition_count.setter
    def condition_count(self, condition_count):
        """
        Sets the condition_count of this SyncProgress.

        :param condition_count: The condition_count of this SyncProgress.
        :type: int
        """

        self._condition_count = condition_count

    @property
    def condition_progress(self):
        """
        Gets the condition_progress of this SyncProgress.

        :return: The condition_progress of this SyncProgress.
        :rtype: int
        """
        return self._condition_progress

    @condition_progress.setter
    def condition_progress(self, condition_progress):
        """
        Sets the condition_progress of this SyncProgress.

        :param condition_progress: The condition_progress of this SyncProgress.
        :type: int
        """

        self._condition_progress = condition_progress

    @property
    def relationship_count(self):
        """
        Gets the relationship_count of this SyncProgress.

        :return: The relationship_count of this SyncProgress.
        :rtype: int
        """
        return self._relationship_count

    @relationship_count.setter
    def relationship_count(self, relationship_count):
        """
        Sets the relationship_count of this SyncProgress.

        :param relationship_count: The relationship_count of this SyncProgress.
        :type: int
        """

        self._relationship_count = relationship_count

    @property
    def relationship_progress(self):
        """
        Gets the relationship_progress of this SyncProgress.

        :return: The relationship_progress of this SyncProgress.
        :rtype: int
        """
        return self._relationship_progress

    @relationship_progress.setter
    def relationship_progress(self, relationship_progress):
        """
        Sets the relationship_progress of this SyncProgress.

        :param relationship_progress: The relationship_progress of this SyncProgress.
        :type: int
        """

        self._relationship_progress = relationship_progress

    @property
    def scalar_count(self):
        """
        Gets the scalar_count of this SyncProgress.

        :return: The scalar_count of this SyncProgress.
        :rtype: int
        """
        return self._scalar_count

    @scalar_count.setter
    def scalar_count(self, scalar_count):
        """
        Sets the scalar_count of this SyncProgress.

        :param scalar_count: The scalar_count of this SyncProgress.
        :type: int
        """

        self._scalar_count = scalar_count

    @property
    def scalar_progress(self):
        """
        Gets the scalar_progress of this SyncProgress.

        :return: The scalar_progress of this SyncProgress.
        :rtype: int
        """
        return self._scalar_progress

    @scalar_progress.setter
    def scalar_progress(self, scalar_progress):
        """
        Sets the scalar_progress of this SyncProgress.

        :param scalar_progress: The scalar_progress of this SyncProgress.
        :type: int
        """

        self._scalar_progress = scalar_progress

    @property
    def signal_count(self):
        """
        Gets the signal_count of this SyncProgress.

        :return: The signal_count of this SyncProgress.
        :rtype: int
        """
        return self._signal_count

    @signal_count.setter
    def signal_count(self, signal_count):
        """
        Sets the signal_count of this SyncProgress.

        :param signal_count: The signal_count of this SyncProgress.
        :type: int
        """

        self._signal_count = signal_count

    @property
    def signal_progress(self):
        """
        Gets the signal_progress of this SyncProgress.

        :return: The signal_progress of this SyncProgress.
        :rtype: int
        """
        return self._signal_progress

    @signal_progress.setter
    def signal_progress(self, signal_progress):
        """
        Sets the signal_progress of this SyncProgress.

        :param signal_progress: The signal_progress of this SyncProgress.
        :type: int
        """

        self._signal_progress = signal_progress

    @property
    def user_group_count(self):
        """
        Gets the user_group_count of this SyncProgress.

        :return: The user_group_count of this SyncProgress.
        :rtype: int
        """
        return self._user_group_count

    @user_group_count.setter
    def user_group_count(self, user_group_count):
        """
        Sets the user_group_count of this SyncProgress.

        :param user_group_count: The user_group_count of this SyncProgress.
        :type: int
        """

        self._user_group_count = user_group_count

    @property
    def user_group_progress(self):
        """
        Gets the user_group_progress of this SyncProgress.

        :return: The user_group_progress of this SyncProgress.
        :rtype: int
        """
        return self._user_group_progress

    @user_group_progress.setter
    def user_group_progress(self, user_group_progress):
        """
        Sets the user_group_progress of this SyncProgress.

        :param user_group_progress: The user_group_progress of this SyncProgress.
        :type: int
        """

        self._user_group_progress = user_group_progress

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        if not isinstance(other, SyncProgress):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



from __future__ import annotations

import os
from distutils import dir_util

from seeq.base import system
from seeq.spy import _common
from seeq.spy._errors import *


def copy(folder=None, *, overwrite=False, advanced=True):
    """
    Copies the SPy Documentation (Jupyter Notebooks) to a particular folder. This is typically used when the seeq
    module is installed via PyPI.

    This function should be called again with overwrite=True if the seeq module is updated.

    Parameters
    ----------
    folder : str
        The folder to receive the documentation. By default it will be copied to a 'SPy Documentation' folder in the
        current working directory.

    overwrite : bool
        If True, any existing files in the specified folder will be deleted before the documentation is copied in.

    advanced : bool
        Deprecated.
    """
    _common.validate_argument_types([
        (folder, 'folder', str),
        (overwrite, 'overwrite', bool),
        (advanced, 'advanced', bool)
    ])

    if not folder:
        folder = os.path.join(os.getcwd(), 'SPy Documentation')

    folder = system.cleanse_path(folder)

    if os.path.exists(folder):
        if not overwrite:
            raise SPyRuntimeError('The "%s" folder already exists. If you would like to overwrite it, supply the '
                                  'overwrite=True parameter. Make sure you don\'t have any of your own work in that '
                                  'folder!' % folder)

        dir_util.remove_tree(folder)

    library_doc_folder = system.cleanse_path(os.path.join(os.path.dirname(__file__), 'Documentation'))

    dir_util.copy_tree(library_doc_folder, folder)

    print('Copied SPy library documentation to "%s"' % folder)
